package monitor

import (
	"time"

	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/mdb"
	"shylinux.com/x/icebergs/base/nfs"
	"shylinux.com/x/icebergs/base/web"
	"shylinux.com/x/icebergs/core/code"
	kit "shylinux.com/x/toolkits"

	"2025-dev.shylinux.com/x/20250215-cluster/src/travel"

	"github.com/shirou/gopsutil/v4/cpu"
	"github.com/shirou/gopsutil/v4/disk"
	"github.com/shirou/gopsutil/v4/mem"
	"github.com/shirou/gopsutil/v4/net"
)

type monitor struct {
	travel.Travel
	fields string `data:"version,time,Total,Free,Available,Used,UsedPercent,ModelName,Mhz,Core,Thread,CorePercent,ThreadPercent,DiskTotal,DiskFree,DiskUsed,DiskUsedPercent,BytesRecv,BytesSent,PacketsRecv,PacketsSent"`
	list   string `name:"list space auto" help:"设备监控"`
}

func (s monitor) List(m *ice.Message, arg ...string) {
	if len(arg) == 0 {
		c, _ := cpu.Info()
		v, _ := mem.VirtualMemory()
		d, _ := disk.Usage(kit.Pwd())
		n, _ := net.IOCounters(false)
		kit.For(kit.Split(m.OptionDefault(mdb.FIELDS, m.Config(mdb.FIELDS))), func(field string) {
			switch field {
			case nfs.VERSION:
				m.Push(field, ice.Info.Make.Version)
			case mdb.TIME:
				m.Push(field, ice.Info.Make.Time)
			case "ModelName":
				m.Push(field, c[0].ModelName)
			case "Mhz":
				list := []string{}
				for _, v := range c {
					list = append(list, kit.Format("%.2f", v.Mhz))
				}
				m.Push(field, kit.Join(list, ","))
			case "Core", "Thread":
				n, _ := cpu.Counts(field == "Thread")
				m.Push(field, n)
			case "CorePercent", "ThreadPercent":
				break
				p, _ := cpu.Percent(3*time.Second, field == "ThreadPercent")
				list := []string{}
				for _, v := range p {
					list = append(list, kit.Format("%.2f", v))
				}
				m.Push(field, kit.Join(list, ","))
			case "Total":
				m.Push(field, kit.FmtSize(v.Total))
			case "Free":
				m.Push(field, kit.FmtSize(v.Free))
			case "Available":
				m.Push(field, kit.FmtSize(v.Available))
			case "Used":
				m.Push(field, kit.FmtSize(v.Used))
			case "UsedPercent":
				m.Push(field, kit.Format("%.2f%%", v.UsedPercent))
			case "DiskTotal":
				m.Push(field, kit.FmtSize(d.Total))
			case "DiskFree":
				m.Push(field, kit.FmtSize(d.Free))
			case "DiskUsed":
				m.Push(field, kit.FmtSize(d.Used))
			case "DiskUsedPercent":
				m.Push(field, kit.Format("%.2f%%", d.UsedPercent))
			case "BytesRecv":
				m.Push(field, kit.FmtSize(n[0].BytesRecv))
			case "BytesSent":
				m.Push(field, kit.FmtSize(n[0].BytesSent))
			case "PacketsRecv":
				m.Push(field, kit.FmtSize(n[0].PacketsRecv))
			case "PacketsSent":
				m.Push(field, kit.FmtSize(n[0].PacketsSent))
			default:
				m.Push(field, "")
			}
		})
		m.Action(s.Foreach)
		m.Cmdy(code.PUBLISH, nfs.BINARY)
	} else {
		m.Cmdy(web.SPACE, arg[0], m.PrefixKey())
	}
}
func (s monitor) Foreach(m *ice.Message, arg ...string) {
	s.Travel.Foreach(m, arg...)
	stats := map[string]int{}
	m.Table(func(value ice.Maps) {
		kit.For([]string{"Thread", "Used", "Available", "DiskUsed", "DiskFree"}, func(key string) { stats[key] += kit.Int(value[key]) })
	})
	list := []string{}
	kit.For(stats, func(key string, value int) { list = append(list, key, kit.FmtSize(value)) })
	m.StatusTimeCount(list).Sort("Available,space", ice.INT)
}

func init() { ice.Cmd("web.chat.dev.machine.monitor", monitor{}) }
